#!/usr/bin/python
#
from __future__ import division
import time
import math
import json
import time
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication

version = 1.0


def acquire_2k_histogram(dwell_time, file_name):
    """
    We assume that only one MCA-1000 is attached, and that the MCA Data Server is running.
    This function acquires a 2Kx32 histogram plus count rate data.  Notice the histo_2k parameter
    in the command (cmd).  In 2K mode, there is only one histogram bank, and background subtraction
    is not available.  Also notice "num_items": 2064 to read 16 + 2048 data values.

    :param dwell_time: Acquisition time in seconds
    :param file_name: Write the histogram to this file as a on-line json file
    :return: None
    """
    mds_client, sn = bpi_communication.connect_one_device()

    # Start new sample histogram acquisition
    cmd = {"type": "mca_cmd", "name": "arm_ctrl",  "dir": "rmw", "sn": sn,
           "data": {"fields": {"run_time_sample": dwell_time},
                    "user":{"histogram_run": 1, "histo_2k": 1,"clear_statistics": 1, "clear_histogram": 1}}
          }
    ret = mds_client.sar(cmd)

    time.sleep(dwell_time+1)  # wait

    # Read count rate and histogram data
    cmd = {"type": "mca_cmd", "name": "arm_histogram", "dir": "read", "num_items": 2064, "sn": sn}
    ret = mds_client.sar(cmd)
    mca_data = ret[sn]["fields"]

    with open(file_name, 'w') as fout:
        fout.write(json.dumps(mca_data) + "\n")


acquire_2k_histogram(dwell_time=30, file_name="../data/2K_histo.json")

