import json
import time
from datetime import datetime
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication


def record_sample_counting(par):
    """
    We assume that only one device is attached, and that the device_server is running.

    1) Preparation:
    User has set 'sample-bck' to 1 in the Alarm panel of the PMT-MCA dashboard. Alternatively,
    a previous program has set ['user']['sample_alarm'] to 1.  And the user has acquired
    a background histogram for some time.  The PMT-MCA has not been turned off, and the
    background histogram is still stored in the device.

    2) Sample measurement:
    The function will start a new sample acquisition run and record data once per 'dt' seconds,
    for a total of 'num_records' data points.

    :param file_name: Write the arm_status data to this file.
    :param dt: Time between recordings
    :param num_records: Number of sample records to write to disk.
    :return: None
    """

    file_name = par["file_name"]
    dwell_time = par["dwell_time"]
    num_records = par["num_records"]

    mds_client, sn = bpi_communication.connect_one_device()

    # Start the sample acquisition run
    cmd = {"type": "mca_cmd", "name": "arm_ctrl", "dir": "rmw", "sn": sn,
           "data": {"fields": {"run_time": 0},  # No time out
                    "user": {'sample_alarm': 1, 'clear_statistics': 1, 'clear_histogram': 1, 'histogram_run': 1, 'active_bank': 0}}}
    ret = mds_client.sar(cmd)

    # Delete previous data
    with open(file_name, 'w') as fout:
        pass

    for n in range(num_records):
        time.sleep(dwell_time)
        # Read back data and store
        # In the data file, look for fields with names starting with 'roi'
        cmd = {"type": "mca_cmd", "name": "arm_status", "dir": "read", "sn": sn}
        ret = mds_client.sar(cmd)
        arm_status = ret[sn]
        with open(file_name, 'a') as fout:
            fout.write(json.dumps(arm_status) + "\n")


time_stamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
par = {
    "file_name": "../data/sample_{}.dat".format(time_stamp),
    "dwell_time": 10, # in seconds
    "num_records": 30
}
record_sample_counting(par)
