import json
import time
import zmq
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication as com


def write_arm_ctrl(par):
    """
    We assume that only one MCA is attached, and that the MCA Data Server is running.
    Read an arm_ctrl settings file and program it into the MCA

    :param par["file_name"]: Read arm_ctrl data from this file
    :param par["to_nvmem"]: 1-> also write the settings into non-volatile memory.
    :return:
    """

    mds_ip = "tcp://127.0.0.1:5507"
    mds_client = com.zmq_device(mds_ip, "client")

    cmd = {"type": "server_cmd", "name": "hello"}
    ret = mds_client.sar(cmd)
    sn_list = ret['sn_list']
    for sn in sn_list:
        print("Serial number:", sn)
    sn = sn_list[0]  # We only use one attached device here

    # Read the arm_ctrl structure from a json file;
    with open(par["file_name"], "r") as fin:
        arm_ctrl = json.loads(fin.read())["arm_ctrl"]
        
    # Write to MCA
    cmd = {"type": "mca_cmd", "name": "arm_ctrl", "dir": "write", "sn": sn, 
           "data": arm_ctrl}
    mds_client.sar(cmd)
    
    # Write to non-volatile memory in the MCA
    if par["to_nvmem"]:
        cmd = {"type": "mca_cmd", "name": "arm_ctrl", "dir": "write", "memory": "flash", "sn": sn, 
               "data": arm_ctrl}
        mds_client.sar(cmd)

    # Read back and store
    cmd = {"type": "mca_cmd", "name": "arm_ctrl", "dir": "read", "sn": sn}
    ret = mds_client.sar(cmd)
    arm_ctrl = ret[sn]
    with open(par["read_back_file_name"], 'w') as fout:
        fout.write(json.dumps(arm_ctrl) + "\n")

par = {
    "file_name": "../data/arm_ctrl.json",
    "read_back_file_name": "../data/arm_ctrl_read_back.json",
    "to_nvmem": 1
}
write_arm_ctrl(par)
