import matplotlib.pyplot as plt

"""
    This is a collection of a few simple line plots, to be used together with a plot_controls.py file.
    The plots rely on matplotlib and, indirectly, numpy.
    They are used to visaulize the data acquired in the MCA example codes.
"""

def line_plot(x_data, y_data, ctrl):
    """
        Single y(x) line plot or scatter plot;
        and y_data = [y0, y1, y2, ...]
        and x_data = [x0, x1, x2, ...]
    """

    fig, ax = plt.subplots()
    lp1 = ax.plot(x_data, y_data)
    plt.setp(lp1, **ctrl['line_ctrl'])


    ax.set_xlabel(ctrl['labels']['xlabel'] , **ctrl['labels']['xlabel_ctrl'])
    ax.set_ylabel(ctrl['labels']['ylabel'], **ctrl['labels']['ylabel_ctrl'])
    ax.set_title(ctrl['labels']['title'], **ctrl['labels']['title_ctrl'])
    ax.grid(**ctrl['grid_ctrl'])
    ax.set(**ctrl['axis_ctrl'])

    """
    bbox_args = dict(boxstyle="round", pad=0.5, ec="#555555", fc=(0.9, 0.9, 0.9, 0.9), linewidth=2)  # fill, bounds(l,b,w,h) are ignored
    comment = ""  # Empty string nixes the display
    ax.annotate(comment, fontsize=10,
                xycoords='axes fraction', xy=(0.35, 0.90), ha="left", va="bottom", bbox=bbox_args)
    """


def line_plot_y_array(x_data, y_array, ctrl):
    """
        Plot an array of y_data vs a single set of x-data
        y_array = [y_data_0, y_data_1, ...]
        and y_data_0 = [y0, y1, y2, ...]

        and x_data = [x0, x1, x2, ...]
    """

    fig, ax = plt.subplots()
    lp_list = []
    colors = ["SteelBlue", "Turquoise", "LimeGreen", "OrangeRed", "FireBrick"]
    markerfacecolors = ["SteelBlue", "Turquoise", "LimeGreen", "OrangeRed", "FireBrick"]
    for nd, y_data in enumerate(y_array):
        lp_list += ax.plot(x_data, y_data)
        ctrl['line_ctrl']['color'] = colors[nd%len(colors)]
        ctrl['line_ctrl']['markerfacecolor'] = markerfacecolors[nd%len(markerfacecolors)]
        plt.setp(lp_list[-1], **ctrl['line_ctrl'])


    ax.set_xlabel(ctrl['labels']['xlabel'] , **ctrl['labels']['xlabel_ctrl'])
    ax.set_ylabel(ctrl['labels']['ylabel'], **ctrl['labels']['ylabel_ctrl'])
    ax.set_title(ctrl['labels']['title'], **ctrl['labels']['title_ctrl'])
    ax.grid(**ctrl['grid_ctrl'])
    ax.set(**ctrl['axis_ctrl'])

    """
    bbox_args = dict(boxstyle="round", pad=0.5, ec="#555555", fc=(0.9, 0.9, 0.9, 0.9), linewidth=2)  # fill, bounds(l,b,w,h) are ignored
    comment = ""  # Empty string nixes the display
    ax.annotate(comment, fontsize=10,
                xycoords='axes fraction', xy=(0.35, 0.90), ha="left", va="bottom", bbox=bbox_args)
    """


def line_plot_xy_array(x_array, y_array, ctrl):
    """
        Plot an y-data vs x-data where x_array and y_array are lists of data:
        x_array = [x_data_0, x_data_1, ...]
        y_array = [y_data_0, y_data_1, ...]
        and x_data_0 = [x0, x1, x2, ...]
        and y_data_0 = [y0, y1, y2, ...]
    """

    fig, ax = plt.subplots()
    lp_list = []
    colors = ["SteelBlue", "Turquoise", "LimeGreen", "OrangeRed", "FireBrick"]
    markerfacecolors = ["SteelBlue", "Turquoise", "LimeGreen", "OrangeRed", "FireBrick"]
    for nd, x_data, y_data in zip(range(len(x_array)), x_array, y_array):
        lp_list += ax.plot(x_data, y_data)
        ctrl['line_ctrl']['color'] = colors[nd%len(colors)]
        ctrl['line_ctrl']['markerfacecolor'] = markerfacecolors[nd%len(markerfacecolors)]
        plt.setp(lp_list[-1], **ctrl['line_ctrl'])


    ax.set_xlabel(ctrl['labels']['xlabel'] , **ctrl['labels']['xlabel_ctrl'])
    ax.set_ylabel(ctrl['labels']['ylabel'], **ctrl['labels']['ylabel_ctrl'])
    ax.set_title(ctrl['labels']['title'], **ctrl['labels']['title_ctrl'])
    ax.grid(**ctrl['grid_ctrl'])
    ax.set(**ctrl['axis_ctrl'])

    """
    bbox_args = dict(boxstyle="round", pad=0.5, ec="#555555", fc=(0.9, 0.9, 0.9, 0.9), linewidth=2)  # fill, bounds(l,b,w,h) are ignored
    comment = ""  # Empty string nixes the display
    ax.annotate(comment, fontsize=10,
                xycoords='axes fraction', xy=(0.35, 0.90), ha="left", va="bottom", bbox=bbox_args)
    """



"""
Example plot control dictionary:

ctrl = {
    "axis_ctrl": {
        "xscale": "linear",  # {"linear", "log", "symlog", "logit", ...}
        "yscale": "linear",  # {"linear", "log", "symlog", "logit", ...}
        "xlim": None, # None or [xmin, xmax]
        "ylim": None # [0, 2000] # None or [ymin, ymax]
    },
    "grid_ctrl": {
        "alpha": 0.2,
        "linestyle": ":",  # {'-', '--', '-.', ':', '', (offset, on-off-seq), ...}
        "linewidth": 1.0,
        "color": "#0000AA"
    },
    "line_ctrl": {
        "alpha": 0.9,
        "linestyle": "-",  # {'-', '--', '-.', ':', '', (offset, on-off-seq), ...}
        "linewidth": 1.0,
        "color": "SteelBlue",
        "drawstyle": "default",  # {'default', 'steps', 'steps-pre', 'steps-mid', 'steps-post'}
        "fillstyle": 'full',  # {'full', 'left', 'right', 'bottom', 'top', 'none'}; fills the marker
        "markersize": 5,
        "markeredgewidth": 0,
        "markeredgecolor": "SteelBlue",
        "markerfacecolor": "SteelBlue",
        "markerfacecoloralt": "SteelBlue", # Fills opposite of fillstyle
        "marker": "",
        "markevery": None
    },
    "labels":
        {
        "xlabel": "MCA bin",
        "xlabel_ctrl": {"fontsize":12, "fontweight": "normal", "color": "#AA0000", "alpha": 0.8},
        "ylabel": "Frequency",
        "ylabel_ctrl": {"fontsize":12, "fontweight": "normal", "color": "#AA0000", "alpha": 0.8},
        "title": "Energy histogram",
        "title_ctrl": {"fontsize":14, "fontweight": "bold", "color": "#AA00AA", "alpha": 0.9}
        },
    "show": False
}

"""
