import json
import time
import zmq
import sys
sys.path.append("../../../../all_mca/bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication as com


def load_map(par):
    """
    We assume that only one MCA-3000 is attached, and that the MCA Data Server is running.
    Load the 2048->128 energy bin map for compressed histogram in the Impress time slice firmware.
    
    :param par["map_file'] : 2048 integer numbers containing the mapping
    :return:
    """

    mds_ip = "tcp://127.0.0.1:5507"
    mds_client = com.zmq_device(mds_ip, "client")

    cmd = {"type": "server_cmd", "name": "hello"}
    ret = mds_client.sar(cmd)
    sn_list = ret['sn_list']
    for sn in sn_list:
        print("Unique serial number:", sn)
    sn = sn_list[0]  # We only use one attached device here
    
    with open(par['map_file'], 'r') as fin:
        bin_map = json.loads(fin.read())

    # Program FPGA controls to reset the statistics counters and the time_slice machinery.;
    cmd = {"type": "mca_cmd", "name": "fpga_impress_map", "dir": "write", "sn": sn,
           "data": {"registers": bin_map }}
    mds_client.sar(cmd)


def make_example_map(par):
    bin_map = [ min(127, n//16+20) for n in range(2048) ]
    #bin_map = [ min(127,n) for n in range(2048) ] * 2048
    #bin_map = [ 31 ] * 2048
    with open(par['map_file'], 'w') as fout:
        fout.write(json.dumps(bin_map, indent=4))

par={
    'map_file': "../data/histo_map.json"
}


make_example_map(par)
load_map(par)
