import json
import time
import zmq
import communication as com


def load_map(par):
    """
    We assume that only one MCA-3000 is attached, and that the MCA Data Server is running.
    Load the 2048->128 energy bin map for compressed histogram in the Impress time slice firmware.
    
    :param par["map_file'] : 2048 integer numbers containing the mapping
    :return:
    """

    mds_ip = "tcp://127.0.0.1:5507"
    mds_client = com.zmq_device(mds_ip, "client")

    cmd = {"type": "server_cmd", "name": "hello"}
    msg = mds_client.send_and_receive(json.dumps(cmd).encode('utf-8')).decode('utf-8')
    ret = json.loads(msg)
    sn_list = ret['sn_list']
    for sn in sn_list:
        print("Unique serial number:", sn)
    sn = sn_list[0]  # We only use one attached device here
    
    with open(par['map_file'], 'r') as fin:
        map = json.loads(fin.read())

    # Program FPGA controls to reset the statistics counters and the time_slice machinery.;
    cmd = {"type": "mca_cmd", "name": "fpga_impress_time_slice", "dir": "write", "sn": sn,
           "data": {"registers": map }}
    mds_client.send_and_receive(json.dumps(cmd).encode('utf-8')).decode('utf-8')


def make_example_map(par):
    for n in range(2048)
    
    map = [ min(127, n//16+8) for n in range(2048) ]
    with open(par['map_file'], 'w') as fout:
        fout.write(json.dumps(map, indent=4))

par['map_file'] = "./data/histo_map.json"

make_example_map(par)
load_map(par)
